; benutzt:
;   temporr: 
;     r0, r16, r17, r18, r19
;
;   dauerhaft: 


.dseg
twi_status: .byte 1
twi_error: .byte 1

.equ timeout_low = 0x88
.equ timeout_high = 0x13 ;= 5000 

.def twi_repeat = r19

.cseg


;-------------------------------------------------------------------------------------------------------
; warten bis TWINT gesetzt ist, status auslesen 
Twi_wait_int:
  clr r16
  clr r17
  sts twi_status, r16
  clr r0
  clr r1
  inc r1

twi_wait_again:
  add r16, r1
  adc r17, r0
  cpi r16, timeout_low
  brne twi_wait_next
  cpi r17, timeout_high
  brne twi_wait_next
  ldi r16, 0xff
  sts twi_status, r16
  ret

twi_wait_next:
  in r18, TWCR
  andi r18, 0x80
  cpi r18, 0x80
  brne twi_wait_again

  in r18, TWSR
  andi r18, 0xF8
  sts twi_status, r18
 
  ret
; Twi_wait_int endp




;-------------------------------------------------------------------------------------------------------
Twi_init_master:
; in r20 = prescaler
;    r21 = bitrate

    ; TWI Init 
    ldi r16, 0b00000100
    out TWCR, r16   ; erstmal nur TWI aktivieren 
    andi r20, 3
    out TWSR, r20   ; Status und Prescaler Register 
    out TWBR, r21   ; Bit Rate Register, 100kHz 
    
    ret
; Twi_init_master endp






;-------------------------------------------------------------------------------------------------------
; TWI read_bytes 
; holt mehrere Bytes und schliesst die bertragung ab 
; in r20 = anzahl
;    r21 = slave_adresse
;    r30:r31 = pointer in speicher auf daten
; out r20 = status (-1=fehler, 0, alles ok, >0 nicht alle bytes empfangen (rest))
 
Twi_read_byte: 
    clr twi_repeat

twi_rb_neustart0:
    ; Startbedingung 
    ldi r16, 0b11100100
    out TWCR, r16  ; TWINT  

    ; warten bis TWINT gesetzt ist 
    rcall Twi_wait_int
    lds r16, twi_status

    cpi r16, 0xff ; timeout beim warten auf TWINT
    breq twi_rb_nobus

    cpi r16, 0x08            ; start erfolgreich
    breq twi_rb_start_ok
    cpi r16, 0x10            ; repeated start erfolgreich
    breq twi_rb_start_ok

twi_rb_nobus:
    ; Bus belegt, wird er wieder freigegeben 
    ldi r16, 0b10010100
    out TWCR, r16  ; TWINT lschen, STOP senden 
    rcall wait_bus 

    ldi r16, 0b10000100
    out TWCR, r16  ; TWINT lschen, Bus freigeben 
    rcall wait_bus 


twi_rb_neustart:
    inc twi_repeat
    
    cpi twi_repeat, 10
    brne twi_rb_neustart0

    ldi r16, 0b10000100
    out TWCR, r16  ; TWINT lschen, Bus freigeben 

    ldi r20, 0xff
    ret

twi_rb_start_ok:
    ori r21, 1 ; READ-bit setzen
    out TWDR, r21  
    ldi r16, 0b10000100
    out TWCR, r16  ; TWINT lschen, Byte senden 

    ; warten bis TWINT gesetzt ist 
    rcall Twi_wait_int
    lds r16, twi_status

    cpi r16, 0x38  ; kein ACK vom slave auf adresse (oder arbitration lost, nur in multimaster)
    breq twi_rb_nobus     ; bus freigeben und neustart, nach 10 versuchen abbruch

    cpi r16, 0x48  ; adresse bertragen, aber kein ack vom slave
    breq twi_rb_neustart

    cpi r16, 0x40
    brne twi_rb_neustart

    // adresse ok, jetzt bytes empfangen

twi_rb_next_byte:
    ldi r16, 0b10000100         
    cpi r20, 1
    breq twi_rb_noack
    ori r16, 0b01000000   ; ack setzen, solange noch nicht letztes byte
twi_rb_noack:
    out TWCR, r16  ; TWINT lschen, Byte empfangen
    rcall Twi_wait_int
    lds r16, twi_status

    cpi r16, 0x38  ; kein ACK vom slave auf adresse (oder arbitration lost)
    breq twi_rb_nobus     ; bus freigeben und neustart, nach 10 versuchen abbruch
    cpi r16, 0x58
    breq twi_rb_letztes_byte
    cpi r16, 0x50
    brne twi_rb_neustart

    ; byte mit ack empfangen

    in r16, TWDR
    ST Z+, r16
    dec r20
    cpi r20, 0
    breq twi_rb_stop
    rjmp twi_rb_next_byte

twi_rb_letztes_byte:

    in r16, TWDR
    ST Z+, r16
    dec r20

twi_rb_stop:    

  ; STOPbedingung kommt hier immer im Ablauf, egal welcher Status 
  ldi r16, 0b10010100
  out TWCR, r16  ; TWINT lschen, STOP senden 
  ; nach einem STOP wird TWINT nicht mehr gesetzt, 
  ; man darf/kann also nicht darauf warten ! 
  ret




;-------------------------------------------------------------------------------------------------------
; TWI write_bytes 
; schreibt mehrere Bytes und schliesst die bertragung ab 
; in r20 = anzahl
;    r21 = slave_adresse
;    r30:r31 = pointer in speicher auf daten
; out r20 = status (-1=fehler, 0, alles ok, >0 nicht alle bytes geschrieben (rest))
 
Twi_write_byte: 
    clr twi_repeat

twi_wb_neustart0:
    ; Startbedingung 
    ldi r16, 0b10100100
    out TWCR, r16  ; TWINT  

    ; warten bis TWINT gesetzt ist 
    rcall Twi_wait_int
    lds r16, twi_status

    cpi r16, 0xff ; timeout beim warten auf TWINT
    breq twi_wb_nobus

    cpi r16, 0x08            ; start erfolgreich
    breq twi_wb_start_ok
    cpi r16, 0x10            ; repeated start erfolgreich
    breq twi_wb_start_ok

twi_wb_nobus:
    ; Bus belegt, wird er wieder freigegeben 
    ldi r16, 0b10000100
    out TWCR, r16  ; TWINT lschen, Bus freigeben 
    rcall wait_bus 

twi_wb_neustart:
    inc twi_repeat
    cpi twi_repeat, 10
    brne twi_wb_neustart0

    ldi r16, 0b10000100
    out TWCR, r16  ; TWINT lschen, Bus freigeben 

    ldi r20, 0xff
    ret

twi_wb_start_ok:
    andi r21, 0xfe ; READ-bit lschen
    out TWDR, r21  
    ldi r16, 0b10000100
    out TWCR, r16  ; TWINT lschen, Byte senden 

    ; warten bis TWINT gesetzt ist 
    rcall Twi_wait_int
    lds r16, twi_status

    cpi r16, 0x38  ; kein ACK vom slave auf adresse (oder arbitration lost)
    breq twi_wb_nobus     ; bus freigeben und neustart, nach 10 versuchen abbruch

    cpi r16, 0x20  ; adresse bertragen, aber kein ack vom slave
    breq twi_wb_neustart

    cpi r16, 0x18
    brne twi_wb_neustart

    // adresse ok, jetzt bytes senden

twi_wb_next_byte:
    ld r16, Z+
    out TWDR, r16
    ldi r16, 0b10000100         
    cpi r20, 1
    brne twi_wb_noack
    ori r16, 0b01000000   ; ack setzen, solange noch nicht letztes byte
twi_wb_noack:
    out TWCR, r16  ; TWINT lschen, Byte empfangen
    rcall Twi_wait_int
    lds r16, twi_status

    cpi r16, 0x30
    breq twi_wb_letztes_byte
    cpi r16, 0x28
    brne twi_wb_neustart

    ; byte mit ack gesendet
    dec r20
    cpi r20, 0
    breq twi_wb_stop
    rjmp twi_wb_next_byte

twi_wb_letztes_byte:
    dec r20

twi_wb_stop:    

  ; STOPbedingung kommt hier immer im Ablauf, egal welcher Status 
  ldi r16, 0b10010100
  out TWCR, r16  ; TWINT lschen, STOP senden 
  ; nach einem STOP wird TWINT nicht mehr gesetzt, 
  ; man darf/kann also nicht darauf warten ! 
  ret




;-------------------------------------------------------------------------------------------------------
wait_bus:
  ; warte 25ms = 92250cl bei 3.69Mhz = 23062 schleifen
  ; 
  ldi r29, 0x5a
  ldi r28, 0x16
wait_bus0:
  sbiw r29:r28, 1  ; 2 cl
  brne wait_bus0   ; 2 cl, solange z=0 

  ret





