.equ LCD_MAX_X         = 15
.equ LCD_MAX_Y         = 3        
          
.equ LCDPORT_DATA_IN   = PINA      
.equ LCDPORT_DATA      = PORTA    ; dataport to lcd
.equ LCDPORT_DATA_DIR  = DDRA     ; direction of lcd data port
.equ LCDPORT_CONTR     = PORTD    ; control port to lcd (only bit 0, 1 and 2)
.equ LCDPORT_CONTR_DIR = DDRD     ; direction of control port
  
.equ LCD_P_RS          = 3
.equ LCD_P_RW          = 2
.equ LCD_P_E           = 0


.def LCD_temp          = r25
; benutzte register: r0-r4


; wait values 
;calculated for 3.69 MHz clock cycletime = 271ns
.equ WAIT_15ms_VALUE_LOW       = 203  ; 15ms =
.equ WAIT_15ms_VALUE_HIGH      = 9;1   ; 91,203 => 55699 cl = 15094,6s 
.equ WAIT_4ms1_VALUE_LOW       = 196  ; 4.1ms 
.equ WAIT_4ms1_VALUE_HIGH      = 2;6   ; 26,196 => 15373 cl = 4166,1s
.equ WAIT_100micro_VALUE_LOW   = 20   ; 100s 
.equ WAIT_100micro_VALUE_HIGH  = 0;6    ; 6,20 => 385 cl = 104,336s
        
	
; after setup of rs and rw wait 40ns before raising E	
; hold data for 60 ns before falling E and 10ns after E falling
; hold RW and RS 10ns after falling E
; min. cycle time 500ns
; hold E at least for 220ns
; timings for 1 cycle
; 4.00 Mhz   250ns 
; 3.69 Mhz	 271ns
; 2.00 Mhz   500ns
; 1.84 Mhz	 543ns
; 1.23 Mhz	 813ns
; 1.00 Mhz  1000ns 
; 32.7 kHz 30581ns

;--------------------------------------------------------------
; init of lcd ports
_LCD_init_display:
  ; set all control bits to output-bits
  sbi LCDPORT_CONTR_DIR, LCD_P_RS
  sbi LCDPORT_CONTR_DIR, LCD_P_E
  sbi LCDPORT_CONTR_DIR, LCD_P_RW

  cbi LCDPORT_CONTR, LCD_P_RS
  cbi LCDPORT_CONTR, LCD_P_E
  cbi LCDPORT_CONTR, LCD_P_RW

  ; set dataport to output for init.
  ser LCD_temp
  out LCDPORT_DATA_DIR, LCD_temp
  
  ; delay 15ms
  ldi r21, WAIT_15ms_VALUE_HIGH
  ldi r20, WAIT_15ms_VALUE_LOW
  rcall _wait 

  cbi LCDPORT_CONTR, LCD_P_RS   ; LCD_p_rs = off
  cbi LCDPORT_CONTR, LCD_P_RW   ; LCD_p_rw = off
  ldi LCD_temp, $30
  out LCDPORT_DATA, LCD_temp	; LCD_dat = &H30 
  nop
  sbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = on
  nop
  cbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = off

  ; delay 4.1 ms 
  ldi r21, WAIT_4ms1_VALUE_HIGH
  ldi r20, WAIT_4ms1_VALUE_LOW
  rcall _wait 


  cbi LCDPORT_CONTR, LCD_P_RS   ; LCD_p_rs = off
  cbi LCDPORT_CONTR, LCD_P_RW   ; LCD_p_rw = off
  ldi LCD_temp, $30
  out LCDPORT_DATA, LCD_temp	; LCD_dat = &H30
  nop
  sbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = on
  nop
  cbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = off
  
  ; delay 100s 
  ldi r21, WAIT_100micro_VALUE_HIGH
  ldi r20, WAIT_100micro_VALUE_LOW
  rcall _wait 

  cbi LCDPORT_CONTR, LCD_P_RS   ; LCD_p_rs = off
  cbi LCDPORT_CONTR, LCD_P_RW   ; LCD_p_rw = off
  ldi LCD_temp, $30
  out LCDPORT_DATA, LCD_temp	; LCD_dat = &H30
  nop
  sbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = on
  nop
  cbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = off
  
  ; delay 100s 
  ldi r21, WAIT_100micro_VALUE_HIGH
  ldi r20, WAIT_100micro_VALUE_LOW
  rcall _wait 

  cbi LCDPORT_CONTR, LCD_P_RS   ; LCD_p_rs = off
  cbi LCDPORT_CONTR, LCD_P_RW   ; LCD_p_rw = off
  ldi LCD_temp, 0b00111000		; 2 zeilen: 0b00111000
  out LCDPORT_DATA, LCD_temp	
  nop
  sbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = on
  nop
  cbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = off

  ldi r20, 0b00001000			; display off  
  rcall write_byte_to_lcd

  ldi r20, 0b00000001			; display clear
  rcall write_byte_to_lcd

  ldi r20, 0b00000110			; entrymode
  rcall write_byte_to_lcd

  rcall waitforbusy

  ret

 
  
write_byte_to_lcd:   ; input r20
  rcall waitforbusy

  ser LCD_temp
  out LCDPORT_DATA_DIR, LCD_temp
  cbi LCDPORT_CONTR, LCD_P_RS   ; LCD_p_rs = off
  cbi LCDPORT_CONTR, LCD_P_RW   ; LCD_p_rw = off
  out LCDPORT_DATA, r20     	
  nop
  sbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = on
  nop
  cbi LCDPORT_CONTR, LCD_P_e    ; LCD_p_e = off
  ; rcall waitforbusy
  ret


waitforbusy:
;  ldi r21, WAIT_15ms_VALUE_HIGH
;  ldi r20, WAIT_15ms_VALUE_LOW
;  rcall _wait 
;ret

  clr LCD_temp
;  out LCDPORT_DATA, LCD_temp
  out LCDPORT_DATA_DIR, LCD_temp ; dataport as input
  nop 

_wfb_again:
  cbi LCDPORT_CONTR, LCD_P_RS   ; LCD_p_rs = off
  sbi LCDPORT_CONTR, LCD_P_RW   ; LCD_p_rw = on
  sbi LCDPORT_CONTR, LCD_P_E    ; LCD_p_e = on
  nop
  in  LCD_temp, LCDPORT_DATA_IN
  cbi LCDPORT_CONTR, LCD_P_E    ; LCD_p_e = on

  sbrc LCD_temp, 7
  rjmp _wfb_again
;  ldi r21, WAIT_15ms_VALUE_HIGH
;  ldi r20, WAIT_15ms_VALUE_LOW
;  rcall _wait 
 
  ser LCD_temp
  out LCDPORT_DATA_DIR, LCD_temp ; dataport as output

  
  ret	
  
   
; wait loop 
; waitcycles: 3 + 4 + 1 + ( (3*r21 - 1) + 1 + 2 + 1) * r20 - 1
; = 7 + ( (3*r20 - 1) + 4) * r21
  
_wait:					; 3 cl for rcall
; input r21, r20
; uses  r31, r30
  mov r31, r21			; 1 cl
_w1:
  mov r30, r20			; 1 cl
_w2:
  dec r30				; 1 cl
  brne _w2				; 2 cl / 1 cl if no branch
  dec r31				; 1 cl
  brne _w1				; 2 cl / 1 cl if no branch
  ret					; 4 cycles



 

  
; write command byte to lcd  
_LCD_write_cmd:
; input r20
  cbi LCDPORT_CONTR, LCD_P_RS
  rjmp lcd_write
  
; write data byte to lcd
_LCD_write_data:
; input r20
  sbi LCDPORT_CONTR, LCD_P_RS

lcd_write:  
  ; set dataport to output
  ser LCD_temp
  out LCDPORT_DATA_DIR, LCD_temp
  
  cbi LCDPORT_CONTR, LCD_P_RW
  sbi LCDPORT_CONTR, LCD_P_E
  nop
  out LCDPORT_DATA, r20
  nop
  cbi LCDPORT_CONTR, LCD_P_E

  rcall waitforbusy
  ret
  
 
;------------------------------------------------------------------------------
; high-level print functions 
 
_LCD_gotoxy:
; in r20 = x, r21 = y
  clr r0
  add r0, r20
  ldi r16, $40
  
  sbrc r21, 0
  add r0, r16           ; add $40, if y=1 or y=3
  ldi r16, (LCD_MAX_X+1) 
  sbrc r21, 1
  add r0, r16 ; add MAXX, if y=0 or y=2

  mov r20, r0
  ori r20, $80
  rcall _LCD_write_cmd
  ret


; print single char (in r20) to current screen pos  
_LCD_print_char:
  rjmp _LCD_write_data
      
  
_LCD_clear:
  ldi r20, 0b00000001
  rjmp _LCD_write_cmd


; divide r21:r20 by 10, result in r21:r20, rest in r22
div10:
  clr r22
  lsl r20
  rol r21
  rol r22
  lsl r20
  rol r21
  rol r22
  lsl r20
  rol r21
  rol r22
  ldi r16, 13
loop10:
  lsl r20
  rol r21
  rol r22
  subi r22, 10
  brlo HB1
  inc r20
  rjmp HB2
HB1:
  subi r22, -10
HB2:
  dec r16
  brne loop10
  ret



; converts a word (r21:r20) to string and prints it to current screen pos  
_LCD_print_word:
; input r21:r20
  rcall div10
  mov r0, r22
  rcall div10
  mov r1, r22
  rcall div10
  mov r2, r22
  rcall div10
  mov r3, r22
  rcall div10
  mov r4, r22

  mov r20, r4
  subi r20, -48
  rcall _LCD_print_char
  mov r20, r3
  subi r20, -48
  rcall _LCD_print_char
  mov r20, r2
  subi r20, -48
  rcall _LCD_print_char
  mov r20, r1
  subi r20, -48
  rcall _LCD_print_char
  mov r20, r0
  subi r20, -48
  rcall _LCD_print_char

  ret




